import { Server } from 'socket.io'

import Logger, { LoggerType } from '#application/logger'
import { TSocket } from '#application/types'
import { Character } from '#entities/character'
import CharacterRepository from '#repositories/characterRepository'

export abstract class BaseEvent {
  protected readonly logger = Logger.type(LoggerType.GAME)

  constructor(
    readonly io: Server,
    readonly socket: TSocket
  ) {}

  protected async getCharacter(): Promise<Character | null> {
    const characterRepository = new CharacterRepository()
    return characterRepository.getById(this.socket.characterId!)
  }

  protected async isCharacterGM(): Promise<boolean> {
    const character = await this.getCharacter()
    return character?.getRole() === 'gm'
  }

  protected emitError(message: string): void {
    this.socket.emit('notification', { title: 'Server message', message })
    this.logger.error('character:connect error', `Player ${this.socket.userId}: ${message}`)
  }

  protected handleError(context: string, error: unknown): void {
    const errorMessage = error instanceof Error ? error.message : String(error)
    this.emitError(`${context}: ${errorMessage}`)
    this.logger.error('character:connect error', errorMessage)
  }
}
