import { BaseEvent } from '#application/base/baseEvent'
import DateManager from '#managers/dateManager'
import CharacterRepository from '#repositories/characterRepository'
import ChatService from '#services/chatService'

type TypePayload = {
  message: string
}

export default class SetTimeCommand extends BaseEvent {
  public listen(): void {
    this.socket.on('chat:message', this.handleEvent.bind(this))
  }

  private async handleEvent(data: TypePayload, callback: (response: boolean) => void): Promise<void> {
    try {
      // Check if command is time
      if (!ChatService.isCommand(data.message, 'time')) return

      // Check if character exists and is GM
      if (!(await this.isCharacterGM())) return

      // Get arguments
      const args = ChatService.getArgs('time', data.message)

      if (!args) {
        return
      }

      const time = args[0] // 24h time, e.g. 17:34

      if (!time) {
        return
      }

      await DateManager.setTime(time)
    } catch (error: any) {
      this.logger.error('command error', error.message)
      callback(false)
    }
  }
}
