import { BaseEvent } from '#application/base/baseEvent'
import { CharacterGender } from '#application/enums'
import { UUID } from '#application/types'
import CharacterHairRepository from '#repositories/characterHairRepository'
import SpriteRepository from '#repositories/spriteRepository'

type Payload = {
  id: UUID
  name: string
  gender: CharacterGender
  isSelectable: boolean
  spriteId: UUID
}

export default class CharacterHairUpdateEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:characterHair:update', this.handleEvent.bind(this))
  }

  private async handleEvent(data: Payload, callback: (success: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const spriteRepository = new SpriteRepository()
      const sprite = await spriteRepository.getById(data.spriteId)
      if (!sprite) return callback(false)

      const characterHairRepository = new CharacterHairRepository()
      const characterHair = await characterHairRepository.getById(data.id)
      if (!characterHair) return callback(false)

      await characterHair.setName(data.name).setGender(data.gender).setIsSelectable(data.isSelectable).setSprite(sprite).setUpdatedAt(new Date()).save()

      return callback(true)
    } catch (error) {
      this.logger.error(`Error updating character hair: ${error instanceof Error ? error.message : String(error)}`)
      return callback(false)
    }
  }
}
