import { BaseEvent } from '#application/base/baseEvent'
import { CharacterType } from '#entities/characterType'
import CharacterTypeRepository from '#repositories/characterTypeRepository'

interface IPayload {}

export default class CharacterTypeListEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:characterType:list', this.handleEvent.bind(this))
  }

  private async handleEvent(data: IPayload, callback: (response: CharacterType[]) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const characterTypeRepository = new CharacterTypeRepository()
      const items = await characterTypeRepository.getAll()
      await characterTypeRepository.getEntityManager().populate(items, ['sprite'])

      return callback(items)
    } catch (error) {
      this.logger.error('gm:characterType:list error', error)
      return callback([])
    }
  }
}
