import { BaseEvent } from '#application/base/baseEvent'
import { ItemType, ItemRarity } from '#application/enums'
import { UUID } from '#application/types'
import ItemRepository from '#repositories/itemRepository'
import SpriteRepository from '#repositories/spriteRepository'

type Payload = {
  id: UUID
  name: string
  description: string
  itemType: ItemType
  stackable: boolean
  rarity: ItemRarity
  spriteId: UUID
}

export default class ItemUpdateEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:item:update', this.handleEvent.bind(this))
  }

  private async handleEvent(data: Payload, callback: (success: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const itemRepository = new ItemRepository()
      const item = await itemRepository.getById(data.id)
      if (!item) return callback(false)

      const spriteRepository = new SpriteRepository()
      const sprite = await spriteRepository.getById(data.spriteId)
      if (!sprite) return callback(false)

      await item.setName(data.name).setDescription(data.description).setItemType(data.itemType).setStackable(data.stackable).setRarity(data.rarity).setSprite(sprite).save()

      return callback(true)
    } catch (error) {
      console.error(error)
      return callback(false)
    }
  }
}
