import { BaseEvent } from '#application/base/baseEvent'
import { UUID } from '#application/types'
import { Sprite } from '#entities/sprite'
import SpriteRepository from '#repositories/spriteRepository'

interface CopyPayload {
  id: UUID
}

export default class SpriteCopyEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:sprite:copy', this.handleEvent.bind(this))
  }

  private async handleEvent(payload: CopyPayload, callback: (success: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const spriteRepository = new SpriteRepository()
      const sourceSprite = await spriteRepository.getById(payload.id)

      if (!sourceSprite) {
        this.logger.error('gm:sprite:copy error', 'Source sprite not found')
        return callback(false)
      }

      // Populate source sprite with spriteActions
      await spriteRepository.getEntityManager().populate(sourceSprite, ['spriteActions'])

      const newSprite = new Sprite()
      await newSprite.setName(`${sourceSprite.getName()} (Copy)`).setSpriteActions(sourceSprite.getSpriteActions()).save()

      return callback(true)
    } catch (error) {
      this.logger.error(`Error copying sprite:`, String(error))
      return callback(false)
    }
  }
}
