import fs from 'fs/promises'

import { BaseEvent } from '#application/base/baseEvent'
import Storage from '#application/storage'
import { Sprite } from '#entities/sprite'

export default class SpriteCreateEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:sprite:create', this.handleEvent.bind(this))
  }

  private async handleEvent(data: undefined, callback: (response: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const public_folder = Storage.getPublicPath('sprites')

      // Ensure the folder exists
      await fs.mkdir(public_folder, { recursive: true })

      const sprite = new Sprite()
      await sprite.setName('New sprite').save()

      const uuid = sprite.id

      // Create folder with uuid
      const sprite_folder = Storage.getPublicPath('sprites', uuid)
      await fs.mkdir(sprite_folder, { recursive: true })

      return callback(true)
    } catch (error) {
      console.error('Error creating sprite:', error)
      return callback(false)
    }
  }
}
