import { BaseEvent } from '#application/base/baseEvent'
import UserRepository from '#repositories/userRepository'

export default class LoginEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('login', this.handleEvent.bind(this))
  }

  private async handleEvent() {
    try {
      if (!this.socket.userId) {
        this.logger.warn('Login attempt without user data')
        return
      }

      const userRepository = new UserRepository()
      this.socket.emit('logged_in', { user: userRepository.getById(this.socket.userId) })
      this.logger.info(`User logged in: ${this.socket.userId}`)
    } catch (error: any) {
      this.logger.error('login error: ' + error.message)
    }
  }
}
