import { BaseRepository } from '#application/base/baseRepository'
import { UUID } from '#application/types'
import { CharacterType } from '#entities/characterType'

class CharacterTypeRepository extends BaseRepository {
  async getFirst() {
    try {
      const repository = this.getEntityManager().getRepository(CharacterType)
      const result = await repository.findOne({ id: { $exists: true } })
      if (result) result.setEntityManager(this.getEntityManager())

      return result
    } catch (error: any) {
      this.logger.error(`Failed to get first character type: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getAll() {
    try {
      const repository = this.getEntityManager().getRepository(CharacterType)
      const results = await repository.findAll()
      for (const result of results) result.setEntityManager(this.getEntityManager())

      return results
    } catch (error: any) {
      this.logger.error(`Failed to get all character types: ${error instanceof Error ? error.message : String(error)}`)
      return []
    }
  }

  async getById(id: UUID) {
    try {
      const repository = this.getEntityManager().getRepository(CharacterType)
      const result = await repository.findOne({ id })
      if (result) result.setEntityManager(this.getEntityManager())

      return result
    } catch (error: any) {
      this.logger.error(`Failed to get character type by ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }
}

export default CharacterTypeRepository
